///////////////////////////////////////////////////////////////////////////////
// ControllerFormGL.cpp
// ====================
// Derived Controller class for OpenGL dialog window
//
//  AUTHOR: Song Ho Ahn (song.ahn@gamil.com)
// CREATED: 2006-07-09
// UPDATED: 2006-08-15
//
//	EXTENSION:
//	AUTHOR: Lukas Chripko (luxas98@gmail.com)
//  UPDATED: 2011-05-23
///////////////////////////////////////////////////////////////////////////////

#include <process.h>                                // for _beginthreadex()
#include "ControllerFormGL.h"
#include "resource.h"
#include "Log.h"
using namespace Win;

#using <mscorlib.dll>
using namespace System;
#using <System.Windows.Forms.dll>
using namespace System::Windows::Forms;
using namespace System::IO;
#using <System.dll>


///////////////////////////////////////////////////////////////////////////////
// default contructor
///////////////////////////////////////////////////////////////////////////////
ControllerFormGL::ControllerFormGL(ModelGL* model, ViewFormGL* view) : model(model), view(view)
{
}



///////////////////////////////////////////////////////////////////////////////
// handle WM_CLOSE
///////////////////////////////////////////////////////////////////////////////
int ControllerFormGL::close()
{
    ::DestroyWindow(handle);
    return 0;
}



///////////////////////////////////////////////////////////////////////////////
// handle WM_CREATE
///////////////////////////////////////////////////////////////////////////////
int ControllerFormGL::create()
{
    // initialize all controls
    view->initControls(handle);

    // place the opengl form dialog in right place, bottome of the opengl rendering window
    //RECT rect = {0, 0, 4, 8};
    //::MapDialogRect(glDialog.getHandle(), &rect);
    //int width = MulDiv(300, 4, rect.right);
    //::SetWindowPos(handle, HWND_TOP, 0, 300, 300, 200, 0);

    return 0;
}



///////////////////////////////////////////////////////////////////////////////
// handle WM_COMMAND
///////////////////////////////////////////////////////////////////////////////
int ControllerFormGL::command(int id, int command, LPARAM msg)
{
    static bool flag = false;

    switch(id)
    {
	case IDC_RESETBUT:
		if(command == BN_CLICKED)
		{
			model->resetDimPos();
		}
		break;
	case IDC_REORDER:
		if(command == BN_CLICKED)
		{
			model->reorder_clutter();
		}
		break;
	case IDC_PCA:
		if(command == BN_CLICKED)
		{
			model->PCA();
		}
		break;
    }
	
    return 0;
}



///////////////////////////////////////////////////////////////////////////////
// handle horizontal scroll notification
///////////////////////////////////////////////////////////////////////////////
int ControllerFormGL::hScroll(WPARAM wParam, LPARAM lParam)
{
    // check if the message comming from trackbar
    HWND trackbarHandle = (HWND)lParam;

    int position = HIWORD(wParam);              // current tick mark position
    if(trackbarHandle)
    {
        // get control ID
        int trackbarId = ::GetDlgCtrlID(trackbarHandle);

        switch(LOWORD(wParam))
        {
        case TB_THUMBTRACK:     // user dragged the slider
            view->updateTrackbars(trackbarHandle, position);
            if(trackbarId == IDC_RED)
                model->setBackgroundRed(position / 255.0f);
            else if(trackbarId == IDC_GREEN)
                model->setBackgroundGreen(position / 255.0f);
            else if(trackbarId == IDC_BLUE)
				model->setBackgroundBlue(position / 255.0f);
			else if(trackbarId == IDC_BLEND)
				model->setRecordAlpha(1.0f - position / 100.0f);
			else if(trackbarId == IDC_THRESHOLD){
				model->setThreshold(position);	
			}
            break;

        case TB_THUMBPOSITION:  // by WM_LBUTTONUP
            break;

        case TB_LINEUP:         // by VK_RIGHT, VK_DOWN
            break;

        case TB_LINEDOWN:       // by VK_LEFT, VK_UP
            break;

        case TB_TOP:            // by VK_HOME
            break;

        case TB_BOTTOM:         // by VK_END
            break;

        case TB_PAGEUP:         // by VK_PRIOR (User click the channel to prior.)
            break;

        case TB_PAGEDOWN:       // by VK_NEXT (User click the channel to next.)
            break;

        case TB_ENDTRACK:       // by WM_KEYUP (User release a key.)
            position = ::SendMessage(trackbarHandle, TBM_GETPOS, 0, 0);
            view->updateTrackbars(trackbarHandle, position);
            if(trackbarId == IDC_RED)
                model->setBackgroundRed(position / 255.0f);
            else if(trackbarId == IDC_GREEN)
                model->setBackgroundGreen(position / 255.0f);
            else if(trackbarId == IDC_BLUE)
                model->setBackgroundBlue(position / 255.0f);
			else if(trackbarId == IDC_BLEND)
				model->setRecordAlpha(1.0f - position / 100.0f);
			else if(trackbarId == IDC_THRESHOLD)
				model->setThreshold(position);
            break;
        }
    }

    return 0;
}
void ControllerFormGL::addListInfo(System::String^ s){
	view->addRecordList(s);
}
void ControllerFormGL::resetListInfo(){
	view->resetRecordList();
}
int ControllerFormGL::resize(int x, int y){
	::SetWindowPos(getHandle(),0, x-15, 0, 450,y,SWP_NOZORDER);
	return 0;
}



