///////////////////////////////////////////////////////////////////////////////
// ModelGL.cpp
// ===========
// Model component of OpenGL
// 
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2006-07-10
// UPDATED: 2006-07-10
//
//	EXTENSION:
//	AUTHOR: Lukas Chripko (luxas98@gmail.com)
//  UPDATED: 2011-05-23
///////////////////////////////////////////////////////////////////////////////

#if _WIN32
#include <windows.h>    // include windows.h to avoid thousands of compile errors even though this class is not depending on Windows
#endif
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <Math.h>
#include "ModelGL.h"
//#include <cpplapack.h>
#include <vector>


#using <mscorlib.dll>
using namespace System;
#using <System.Windows.Forms.dll>
using namespace System::Windows::Forms;
using namespace System::IO;
#using <System.dll>

/*
#include "newmat\include.h"
#include "newmat\newmatap.h"          // newmat headers including advanced functions
#include "newmat\newmatio.h"          // newmat headers including output functions
*/
#include "include.h"
#include "newmat.h"	
#include "newmatap.h"	
#include "newmatio.h"	
#include "newmatnl.h"

#include <msclr\marshal.h> 
using namespace std;
using namespace msclr::interop;
//#include "glPrintText.cpp"

void printText (float x, float y, float z, char* format, ...)
{
	GLvoid *font_style = GLUT_BITMAP_TIMES_ROMAN_24;
	va_list args;	//  Variable argument list
	int len;		//	String length
	int i;			//  Iterator
	char * text;	//	Text

	//  Initialize a variable argument list
	va_start(args, format);

	//  Return the number of characters in the string referenced the list of arguments.
	//  _vscprintf doesn't count terminating '\0' (that's why +1)
	len = _vscprintf(format, args) + 1; 

	//  Allocate memory for a string of the specified size
	text = (char*)malloc(len * sizeof(char));

	//  Write formatted output using a pointer to the list of arguments
	vsprintf_s(text, len, format, args);

	//  End using variable argument list 
	va_end(args);

	//  Specify the raster position for pixel operations.
	glRasterPos3f (x, y, z);

	//  Draw the characters one by one
    for (i = 0; text[i] != '\0'; i++)
        glutBitmapCharacter(font_style, text[i]);

	//  Free the allocated memory for the string
	free(text);
}
///////////////////////////////////////////////////////////////////////////////
// default ctor
///////////////////////////////////////////////////////////////////////////////
ModelGL::ModelGL() : windowWidth(0), windowHeight(0), mouseLeftDown(false),
                     mouseRightDown(false), changeDrawMode(false), drawMode(0),
                     cameraAngleX(0), cameraAngleY(0), cameraDistance(5),
					 animateFlag(false), bgFlag(0), frameBuffer(0), bufferSize(0)
{
    bgColor[0] = bgColor[1] = bgColor[2] = bgColor[3] = 0;
	activated = false;
	pom_dim = -1;
	record_alpha = 1.0f;
	poc_pridanych = 0;
	perm_pocet = 0;
	dim_size = 0;
}



///////////////////////////////////////////////////////////////////////////////
// destructor
///////////////////////////////////////////////////////////////////////////////
ModelGL::~ModelGL()
{
    // deallocate framebuffer
    delete [] frameBuffer;
    frameBuffer = 0;
}
void ModelGL::activateDimension()
{
	for(int i = 0; i < dim_size; i++){
		if( (!activated) && abs(mouseX - dim_pos_x[i]) < 30 ){
			pom_dim = i;
			//myHandler->Dimensions->at(i)->active = true;
			activated = true;
		} else {
			//myHandler->Dimensions->at(i)->active = false;
		}
	}
}

void ModelGL::deactivateDimension()
{
	activated = false;
	if(pom_dim >=0){
		//myHandler->Dimensions->at(pom_dim)->active = false;
		pom_dim = -1;
	}
}

void ModelGL::activateRecords()
{
	
	for(int i = 0; i < dim_size; i++){
		if((mouseX > dim_pos_x[i] && mouseRecX < dim_pos_x[i])||( mouseX < dim_pos_x[i] && mouseRecX > dim_pos_x[i])){
			for(int j = 0; j < record_size; j++){
				int Y = windowHeight - (double)(windowHeight)*(double)(records[j][i] - min_dim[i])/abs(max_dim[i] - min_dim[i]);
				if((mouseY > Y && mouseRecY < Y)||( mouseY < Y && mouseRecY > Y)){
					active_records[j] = true;
				} else {
					active_records[j] = false;
				}	
			}
		}
	}	
}
void ModelGL::deactivateRecords()
{
	for(int i = 0; i < record_size; i++){
		active_records[i] = false;
	}
}	
void ModelGL::moveActiveDimension()
{
	if(pom_dim >= 0){
		dim_pos_x[pom_dim] = mouseX;
	}
}
void ModelGL::addRecord(System::String ^s)
{
	array<System::String^>^ pom = s->Split(' ');
	double d = -1;
	int poc = 0;
	for(int i = 0; i < pom->Length; i++){
		if(System::Double::TryParse(pom[i],d)){
			if(poc_pridanych < record_size)
				records[poc_pridanych][poc] = d;
			poc++;
		}
	}
	poc_pridanych++;
}
void ModelGL::drawLine(int x1, int y1, int x2, int y2)
{
	glBegin(GL_LINES);
		glVertex3f(x1,y1,0.0f);
		glVertex3f(x2,y2,0.0f);
	glEnd();
}
void ModelGL::drawRectangle(int x1, int y1, int x2, int y2)
{
	glBegin(GL_POLYGON);
		glVertex3f(x1,windowHeight - y1,0.0f);
		glVertex3f(x2,windowHeight - y1,0.0f);
		glVertex3f(x2,windowHeight - y2,0.0f);
		glVertex3f(x1,windowHeight - y2,0.0f);
	glEnd();
}
void ModelGL::drawDimensions()
{
	updateDimension();
	
	for(int i = 0; i < dim_size;i++){
		if(i == pom_dim){
			glColor4f(1.0f,0.0f,0.0f,1.0f);
		} else {
			glColor4f(1.0f,1.0f,1.0f,1.0f);
		}
		drawLine(dim_pos_x[i],0,dim_pos_x[i],700);
		glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
		printText(dim_pos_x[i],400,0.0f, (char*)name_dim[i].c_str());
		/*if(i < dim_size -1){
			char* buffer = new char[100];
			itoa(outlinersCount(i, i+1),buffer,10);
			printText(dim_pos_x[i],350,0.0f, buffer);
		}*/
		glColor4f(1.0f,1.0f,1.0f,1.0f);
	}
}
void ModelGL::drawRecords()
{
	/*int X1 = 0,X2 = 0,Y1 = 0,Y2 = 0;
	for(int i = 0; i < record_size ;i++){
		int test = dim_size;
		glColor4f(1.0f,1.0f,1.0f,record_alpha);
		if(active_records[i]){
			glColor4f(1.0f,0.0f,0.0f,record_alpha);
		} else {
			glColor4f(1.0f,1.0f,1.0f,record_alpha);
		}
		glBegin(GL_LINE_STRIP);
		for(int j = 0; j < test; j++){
			X1 = dim_pos_x[j];		
			Y1 = ((double)windowHeight)*(double)(records[i][j] - min_dim[j])/abs(max_dim[j] - min_dim[j]);
			glVertex3f(X1,Y1,0.0f);
		}
		glEnd();
	}*/
	int X1 = 0,X2 = 0,Y1 = 0,Y2 = 0;
	for(int i = 0; i < record_size;i++){
		bool ctrl = false;
		glColor4f(1.0f,1.0f,1.0f,record_alpha);
		if(active_records[i]){
			glColor4f(0.0f,1.0f,0.0f,record_alpha);
			ctrl = true;
		} else {
			glColor4f(1.0f,1.0f,1.0f,record_alpha);
			ctrl = false;
		}
		for(int j = 0; j < dim_size - 1; j++){
			if(outliner_record[i][j]){
				glColor4f(1.0f,0.0f,0.0f, record_alpha);
			} else {
				if(!ctrl)
					glColor4f(1.0f,1.0f,1.0f,record_alpha);
				else 
					glColor4f(0.0f,1.0f,0.0f,record_alpha);
			}
			X1 = dim_pos_x[j];
			Y1 = ((double)windowHeight)*(double)(records[i][j] - min_dim[j])/abs(max_dim[j] - min_dim[j]);
			X2 = dim_pos_x[j+1];
			Y2 = ((double)windowHeight)*(double)(records[i][j+1] - min_dim[j+1])/abs(max_dim[j+1] - min_dim[j+1]);
			drawLine(X1,Y1,X2,Y2);
		}


	}
}
void ModelGL::drawMouseRec()
{	
	glColor4f(1.0f,0.2f,0.2f,0.3f);
	if(mouseRightDown){
		drawRectangle( mouseRecX, mouseRecY, mouseX, mouseY);
	}
	glColor4f(1.0f,1.0f,1.0f,1.0f);
}

///////////////////////////////////////////////////////////////////////////////
// initialize OpenGL states and scene
///////////////////////////////////////////////////////////////////////////////
void reshape(int x, int y){


}
void ModelGL::init()
{
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_DEPTH_TEST);
	//glutReshapeFunc(reshape);
	opened = false;
	glColor4f(1.0f, 1.0f, 1.0f,record_alpha); //Line colour RGB
	
}


///////////////////////////////////////////////////////////////////////////////
// set camera position and lookat direction
///////////////////////////////////////////////////////////////////////////////
void ModelGL::setCamera(float posX, float posY, float posZ, float targetX, float targetY, float targetZ)
{
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(posX, posY, posZ, targetX, targetY, targetZ, 0, 1, 0); // eye(x,y,z), focal(x,y,z), up(x,y,z)
}
///////////////////////////////////////////////////////////////////////////////
// configure projection and viewport
///////////////////////////////////////////////////////////////////////////////
void ModelGL::setViewport(int w, int h)
{
    // assign the width/height of viewport
	//int Width_dif = windowWidth - w;
	//int vzd_dim_dif = 0;
	//if(dim_size > 0)
	//	vzd_dim_dif = Width_dif/dim_size;
    windowWidth = w;
    windowHeight = h;
    bufferSize = w * h * 4; // rgba
    frameBuffer = new unsigned char [bufferSize];
	if(dim_size)
		vzd_dim = windowWidth/dim_size;
    // set viewport to be the entire window
    glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	if(opened)
		for(int i = 0; i < dim_size; i++){
			dim_pos_x[i] = 25 + i*vzd_dim;
		}
		/*for(int i = 0; i < dim_size; i++){
		dim_pos_x[i] = dim_pos_x[i] - vzd_dim_dif;
		dim_pos_x_init[i] = dim_pos_x_init[i] - vzd_dim_dif;
	}*/
    // set perspective viewing frustum
    float aspectRatio = (float)w / h;
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	glOrtho(0.0, w, 0.0, h, -1.0, 1.0);
	//gluPerspective(60, (GLfloat)w / (GLfloat)h, 1.0, 100.0);
	gluPerspective(0,(GLfloat)w/(GLfloat)h,-1.0,1.0);
    // switch to modelview matrix in order to set scene
    glMatrixMode(GL_MODELVIEW);
}



///////////////////////////////////////////////////////////////////////////////
// draw 2D/3D scene
///////////////////////////////////////////////////////////////////////////////
void ModelGL::draw()
{
    // clear buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
    // save the initial ModelView matrix before modifying ModelView matrix
   // glPushMatrix();
	
	//printText(250,250,0, "Ahoj");
	if(opened){
		
		drawing = true;
		drawDimensions();
		drawRecords();
		drawMouseRec();
		//drawOutliners();
		//updateDimension();
	}
	drawing = false;
    //glPopMatrix();
    // check if background colour was changed
    if(bgFlag)
    {
        glClearColor(bgColor[0], bgColor[1], bgColor[2], bgColor[3]);
        bgFlag = false;
    }
}
void ModelGL::setMousePosition(int x, int y)
{
	mouseX = x;
	mouseY = y;
}

///////////////////////////////////////////////////////////////////////////////
// change background colour, the value should be between 0 and 1
///////////////////////////////////////////////////////////////////////////////
void ModelGL::setBackgroundRed(float value)
{
    if(bgColor[0] != value)
    {
        bgColor[0] = value;
        bgFlag = true;
    }
}
void ModelGL::setBackgroundGreen(float value)
{
    if(bgColor[1] != value)
    {
        bgColor[1] = value;
        bgFlag = true;
    }
}
void ModelGL::setBackgroundBlue(float value)
{
    if(bgColor[2] != value)
    {
        bgColor[2] = value;
        bgFlag = true;
    }
}
void ModelGL::setRecordAlpha(float value)
{
	record_alpha = value;
}
void ModelGL::resetDimension(int i)
{
	dim_pos_x[i] = dim_pos_x_init[i];
	//changeDimPosition(0, dim_size - 1);
}
void ModelGL::updateDimension()
{
	if((pom_dim == 0)&&(dim_size > 2)&&(abs(dim_pos_x[pom_dim] - dim_pos_x[pom_dim + 1]) < 20)){
		changeDimPosition(pom_dim,pom_dim+1);
		resetDimension(pom_dim);
	}
	if((pom_dim > 0)&&(dim_size > 2)&&(pom_dim < dim_size - 1)){
		if(abs(dim_pos_x[pom_dim] - dim_pos_x[pom_dim+1]) < 20){
			changeDimPosition(pom_dim,pom_dim+1);
			resetDimension(pom_dim);
			//resetDimPos();
		} else if(abs(dim_pos_x[pom_dim]-dim_pos_x[pom_dim-1]) < 20){
			changeDimPosition(pom_dim,pom_dim-1);
			resetDimension(pom_dim);
			//pom_dim--;		
		}
	}
	if((pom_dim == dim_size-1) && (dim_size > 2) && (abs(dim_pos_x[pom_dim] - dim_pos_x[pom_dim-1]) < 20)){
		changeDimPosition(pom_dim,pom_dim-1);
		resetDimension(pom_dim);
	}
}
void ModelGL::changeDimPosition(int i, int j)
{	
	pom_dim = j;
	int pom_dim_pos_x = 0;
	int pom_dim_pos_x_init = 0;
	float pom_dim_min = 0.0f;
	float pom_dim_max = 0.0f;
	std::string pom_dim_name = "";

	//pom_dim_pos_x = dim_pos_x[i];
	//pom_dim_pos_x_init = dim_pos_x_init[i];

	pom_dim_min = min_dim[i];
	pom_dim_max = max_dim[i];
	pom_dim_name = name_dim[i];

	//dim_pos_x[i] = dim_pos_x[j];
	//dim_pos_x_init[i] = dim_pos_x_init[j];
	//dim_pos_x[i] = dim_pos_x_init[j];
	min_dim[i] = min_dim[j];
	max_dim[i] = max_dim[j];
	name_dim[i] = name_dim[j];

	//dim_pos_x[j] = pom_dim_pos_x;
	//dim_pos_x_init[j] = pom_dim_pos_x_init;
	min_dim[j] = pom_dim_min;
	max_dim[j] = pom_dim_max;
	name_dim[j] = pom_dim_name;
	
	double pom_record_value = 0;
	for(int k = 0; k < this->record_size; k++){
		pom_record_value = records[k][i];
		records[k][i] = records[k][j];
		records[k][j] = pom_record_value;
	}
	fillOutliners();
	resetDimPos();
	
}
void ModelGL::resetDimPos()
{
	for(int i = 0; i < dim_size; i++){
		dim_pos_x[i] = dim_pos_x_init[i];
	}
}

void ModelGL::setThreshold(float t)
{
	if(opened){
		for(int i = 0; i < dim_size; i++){
			threshold[i] = t;
			//threshold[i] = (t/1000000000);
		}
		fillOutliners();
	}
}

bool ModelGL::isOutliner(int record_number, int dim_number1, int dim_number2)
{   
	bool result = false;
	double x1,y1,x2,y2;
		x1 = (double)windowHeight*(double)(records[record_number][dim_number1]- min_dim[dim_number1])/abs(max_dim[dim_number1] - min_dim[dim_number1]);
		y1 = (double)windowHeight*(double)(records[record_number][dim_number2]- min_dim[dim_number2])/abs(max_dim[dim_number2] - min_dim[dim_number2]);
	for(int i = 0; i < record_size; i++){
		x2 = (double)windowHeight*(double)(records[i][dim_number1]- min_dim[dim_number1])/abs(max_dim[dim_number1] - min_dim[dim_number1]);
		y2 = (double)windowHeight*(double)(records[i][dim_number2]- min_dim[dim_number2])/abs(max_dim[dim_number2] - min_dim[dim_number2]);
		if(sqrt(pow(x1 - x2,2) + pow(y1 - y2,2)) < threshold[dim_number1]){
			//double d = 
			//System::Windows::Forms::MessageBox::Show(sqrt(pow(x1 - x2,2) + pow(y1 - y2,2)).ToString());
			result = true;
		} else {
			return false;
		}
	}
	return result;
}
int ModelGL::outlinersCount(int dim1, int dim2)
{
	int count = 0;
	for(int i = 0; i < record_size; i++){
		if(!isOutliner(i,dim1,dim2))
			count++;
	}
	return count;
}
/*
void ModelGL::drawOutliners(){
	for(int j = 0; j < dim_size-1; j++){
		for(int i = 0; i < record_size; i++){
			int y1 = ((double)windowHeight)*(double)(records[i][j] - min_dim[j])/abs(max_dim[j] - min_dim[j]);
			int y2 = ((double)windowHeight)*(double)(records[i][j+1] - min_dim[j+1])/abs(max_dim[j+1] - min_dim[j+1]);
			//
			if(isOutliner(i,j,j+1)){
				glColor3f(1.0f,0.0f,0.0f);
				//System::Windows::Forms::MessageBox::Show( y1.ToString() + " " + y2.ToString() );
			} else{
				glColor3f(1.0f,1.0f,1.0f);
			}
			
			glBegin(GL_LINES);
				glVertex3f(dim_pos_x[j],y1,0.0f);
				glVertex3f(dim_pos_x[j+1],y2,0.0f);	
			glEnd();
		}
	}
}*/
void ModelGL::fillOutliners(){
	for(int i = 0; i < dim_size - 1; i++)
		for(int j = 0; j < record_size; j++){
			outliner_record[j][i] = !isOutliner(j,i,i+1);
		}
}


////////////////////////////////////

void ModelGL::permute(int array[], const int start, const int n){
	//	display the current array
	for (int i=0; i < n; i++)
	{
			permutation[perm_pocet][i] = array[i];
			//System::Windows::Forms::MessageBox::Show(permutation[perm_pocet][i].ToString());
	}
		//std::cout << array[i] << " ";
	perm_pocet++;
	//std::cout << std::endl;
	
	//	start the permutation loop
	if (start < n) {
		int i, j;
		for (i = n-2; i >= start; i--) {
			for (j = i + 1; j < n; j++) {
				//	swap values in positions i and j
				int temp=array[i];
				array[i]=array[j];
				array[j]=temp;
				
				//	perform recursive call to permute
				permute(array, i+1, n);
			}
			
			//	rotate the elements between i and the end of the array
			int tmp = array[i];
			for (int k = i; k < n-1; k++) {
				array[k] = array[k+1];
			}
			array[n-1] = tmp;
		}
	}
}
void ModelGL::reorder_clutter(){
	int faktorial = 1;
	for(int i = 1; i <= dim_size; i++){
		faktorial = faktorial * i;
	}
	int* array = new int[dim_size];
	for(int i = 0; i < dim_size; i++){
		array[i] = i;
	}
	perm_pocet = 0;
	permute(array,0,dim_size);

	for(int i = 0; i < faktorial; i++){
		int sum_outliners = 0;
		for(int j = 0; j < dim_size - 1; j++){
			sum_outliners = sum_outliners + outlinersCount(permutation[i][j],permutation[i][j+1]);
		}
		outliner_numbers[i] = sum_outliners;
		//System::Windows::Forms::MessageBox::Show(outliner_numbers[i].ToString());	
	}
     int min = outliner_numbers[0]; 
	 int num = 0;
     for(int i = 1; i < faktorial; i++)
     {
		 //System::Windows::Forms::MessageBox::Show(outliner_numbers[i].ToString());
		 if(outliner_numbers[i] < min){
                min = outliner_numbers[i];
				num = i;			
		 }
     }
	 //System::Windows::Forms::MessageBox::Show(min.ToString());
	
	 float *max_dim_new = new float[dim_size];
	float *min_dim_new = new float[dim_size];
	std::string *name_dim_new = new string[dim_size];
	//int *dim_number_new = new int[dim_size];
	int *dim_pos_x_new = new int[dim_size];
	int *dim_pos_x_init_new = new int[dim_size];
	 
	for(int i = 0; i < dim_size; i++){
		max_dim_new[i] = max_dim[permutation[num][i]];
		min_dim_new[i] = min_dim[permutation[num][i]];
		name_dim_new[i] = name_dim[permutation[num][i]];
		dim_pos_x_new[i] = dim_pos_x[permutation[num][i]];
		dim_pos_x_init_new[i] = dim_pos_x_init[permutation[num][i]];
	}
	/*System::Windows::Forms::MessageBox::Show(num.ToString());
	System::Windows::Forms::MessageBox::Show(outliner_numbers[num].ToString());
	System::Windows::Forms::MessageBox::Show(permutation[num][0].ToString());
	System::Windows::Forms::MessageBox::Show(permutation[num][1].ToString());
	System::Windows::Forms::MessageBox::Show(permutation[num][2].ToString());*/

	max_dim = max_dim_new;
	min_dim = min_dim_new;
	name_dim = name_dim_new;
	/*for(int i = 0; i < dim_size; i++){
		System::Windows::Forms::MessageBox::Show(max_dim[i].ToString());
		System::Windows::Forms::MessageBox::Show(max_dim_new[i].ToString());
		System::Windows::Forms::MessageBox::Show(min_dim[i].ToString());
		System::Windows::Forms::MessageBox::Show(min_dim_new[i].ToString());
	}*/
	/*for(int i = 0; i < dim_size; i++){
		System::Windows::Forms::MessageBox::Show(permutation[num][i].ToString());
	}
	System::Windows::Forms::MessageBox::Show(outliner_numbers[num].ToString());*/
	//dim_pos_x = dim_pos_x_new;
	//dim_pos_x_init = dim_pos_x_init_new;
	
	double **records_new;

	records_new = (double**)malloc(record_size * sizeof(double*));
	for(int i = 0; i < record_size; i++){
		records_new[i] = (double*)malloc(dim_size * sizeof(double));
	}
	for(int i = 0; i < record_size; i++){
		for(int j = 0; j < dim_size; j++){
			records_new[i][j] = records[i][permutation[num][j]];
		}
	}
	records = records_new;
	setThreshold(threshold[0]);
	//fillOutliners();
	//for(int i = 0; i < record_size; i++
	 //System::Windows::Forms::MessageBox::Show(max_num.ToString());
}
double ModelGL::covariance(int d1,int d2){
	double result = 0.0;
	for(int i = 0; i < record_size; i++){
		result =+ ( records[i][d1] - mean[d1] )*( records[i][d2] - mean[d2] );
	}
	result = result/(record_size - 1);
	return result;
}

double ModelGL::getMax(double **a,int dim){
	 double max = a[0][dim]; 
	 int num = 0;
     for(int i = 0; i < record_size; i++)
     {
		 //System::Windows::Forms::MessageBox::Show(outliner_numbers[i].ToString());
		 if(a[i][dim] > max){
                max = a[i][dim];			
		 }
     }
	 return max;
}
double ModelGL::getMin(double **a,int dim){
	 int min = a[0][dim]; 
     for(int i = 0; i < record_size; i++)
     {
		 //System::Windows::Forms::MessageBox::Show(outliner_numbers[i].ToString());
		 if(a[i][dim] < min){
                min = a[i][dim];			
		 }
     }
	 return min;
}
void ModelGL::PCA(){
	for(int i = 0; i < dim_size; i++){
		double sum = 0;
		for(int j = 0; j < record_size; j++){
			sum += records[j][i];
		}
		mean[i] = sum/record_size;
	}
	ccv = (double**)malloc(dim_size * sizeof(double*));
	for(int i = 0; i < dim_size; i++){
		ccv[i] = (double*)malloc(dim_size * sizeof(double*));
	}
	/*BluebitMatrix31::Matrix^ ccv_m = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ Vec_r = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ Vec_i = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ Val_r = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ Val_i = gcnew BluebitMatrix31::Matrix();*/
	//ccv_m->Size(dim_size,dim_size);
	/*test[0,0] = 25.0;
	double p = test[0,0];
	double d = test[1,1];*/
	SymmetricMatrix ccv_m(dim_size);
	for(int i = 0; i < dim_size; i++){
		for(int j = 0; j < dim_size; j++){
			double c = covariance(i,j);
			ccv_m(i+1,j+1) = c;
			ccv[i][j] = c;
		}
	}
	Matrix eVec;
	//Matrix a;
	DiagonalMatrix eVal;
	//eigenvalues(ccv_m,eVal,eVec);
	EigenValues(ccv_m,eVal,eVec);
	
	//ccv_m->Eigen(Val_r,Val_i,Vec_r,Vec_i);
	eigenvalues = new double[dim_size];
	dim_order_new = new int[dim_size];
	for(int i = 0; i < dim_size; i++){
		dim_order_new[i] = i;
	}
	for(int i = 0; i < eVal.nrows();i++){
		for(int j = 0; j < 1; j++){
				/*marshal_context^ context = gcnew marshal_context();
				char* r;
				r = const_cast<char*>(context->marshal_as<const char*>(Val_r[i,j].ToString()));*/
				eigenvalues[i] = eVec(i+1,j+1);
				//System::Windows::Forms::MessageBox::Show(eigenvalues[i].ToString());
				//Val_r[i,j].ToString()->to;
				//printText(300,300,0.0f,r);
		}
	}
	for(int i = 0; i < dim_size; i++){
		for(int j = dim_size -1 ; j > i; j--){
			if(eigenvalues[j-1] > eigenvalues[j]){
				double swap_double;
				int swap_int;
				swap_double = eigenvalues[j-1];
				swap_int = dim_order_new[j-1];
				eigenvalues[j-1] = eigenvalues[j];
				dim_order_new[j-1] = dim_order_new[j]; 
				eigenvalues[j] = swap_double;
				dim_order_new[j] = swap_int;

				int p = 0;
				p++;
			}
		}
	}

	float *max_dim_new = new float[dim_size];
	float *min_dim_new = new float[dim_size];
	std::string *name_dim_new = new string[dim_size];
	//int *dim_number_new = new int[dim_size];
	int *dim_pos_x_new = new int[dim_size];
	int *dim_pos_x_init_new = new int[dim_size];
	 
	for(int i = 0; i < dim_size; i++){
		max_dim_new[i] = max_dim[dim_order_new[i]];
		min_dim_new[i] = min_dim[dim_order_new[i]];
		name_dim_new[i] = name_dim[dim_order_new[i]];
		dim_pos_x_new[i] = dim_pos_x[dim_order_new[i]];
		dim_pos_x_init_new[i] = dim_pos_x_init[dim_order_new[i]];
	}
	max_dim = max_dim_new;
	min_dim = min_dim_new;
	name_dim = name_dim_new;
	/*
	BluebitMatrix31::Matrix^ vec_T = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ data = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ data_T = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ final_data = gcnew BluebitMatrix31::Matrix();
	BluebitMatrix31::Matrix^ test_M = gcnew BluebitMatrix31::Matrix();
	
	test_M->Size(5,4);

	data->Size(5,4);
	*/
	Matrix vec_T;
	Matrix data(record_size,dim_size);
	Matrix data_T;
	Matrix final_data;
	Matrix test_M;
	
	/*Matrix pom(eVec.nrows(),eVec.ncols());
	for(int i = 0; i < pom.ncols(); i++){
		for(int j = 0; j < pom.nrows(); j++){
			pom(j+1,i+1) = eVec(j+1,dim_order_new[i]+1);
		} 
	}
	eVec = pom;*/
	vec_T = eVec.t();
	//vec_T = Vec_r->Transpose();
	
	for(int i = 0; i < record_size; i++){
		for(int j = 0; j < dim_size; j++){
			data(i+1,j+1) = records[i][j];
		}
	}
	data_T = data.t();
	final_data = vec_T * data_T;
	for(int i = 0; i < record_size; i++){
		for(int j = 0; j < dim_size; j++){
			records[i][j] = final_data(j+1,i+1);
		}
	}
	/*
	float *max_dim_new = new float[dim_size];
	float *min_dim_new = new float[dim_size];
	std::string *name_dim_new = new string[dim_size];
	//int *dim_number_new = new int[dim_size];
	int *dim_pos_x_new = new int[dim_size];
	int *dim_pos_x_init_new = new int[dim_size];
	 
	for(int i = 0; i < dim_size; i++){
		max_dim_new[i] = max_dim[dim_order_new[i]];
		min_dim_new[i] = min_dim[dim_order_new[i]];
		name_dim_new[i] = name_dim[dim_order_new[i]];
		dim_pos_x_new[i] = dim_pos_x[dim_order_new[i]];
		dim_pos_x_init_new[i] = dim_pos_x_init[dim_order_new[i]];
	}
	max_dim = max_dim_new;
	min_dim = min_dim_new;
	name_dim = name_dim_new;
	*/
	double **records_new;

	records_new = (double**)malloc(record_size * sizeof(double*));
	for(int i = 0; i < record_size; i++){
		records_new[i] = (double*)malloc(dim_size * sizeof(double));
	}
	for(int i = 0; i < record_size; i++){
		for(int j = 0; j < dim_size; j++){
			records_new[i][j] = records[i][dim_order_new[j]];
		}
	}
	records = records_new;

	for(int i = 0; i < dim_size; i++){
		max_dim[i] = getMax(records,i);
		min_dim[i] = getMin(records,i);
	}	
}