///////////////////////////////////////////////////////////////////////////////
// ModelGL.h
// =========
// Model component of OpenGL
// 
//  AUTHOR: Song Ho Ahn (song.ahn@gmail.com)
// CREATED: 2006-07-10
// UPDATED: 2006-07-10
//
//	EXTENSION:
//	AUTHOR: Lukas Chripko (luxas98@gmail.com)
//  UPDATED: 2011-05-23
///////////////////////////////////////////////////////////////////////////////

#ifndef MODEL_GL_H
#define MODEL_GL_H

#include <string>

using namespace std;

class ModelGL
{
public:
    ModelGL();
    ~ModelGL();
	/////////////////////////////////////////////////////////
	// EXTENSION
	/////////////////////////////////////////////////////////
	void activateDimension();
	void deactivateDimension();

	void activateRecords();
	void deactivateRecords();

	void moveActiveDimension();
	void addRecord(System::String^ s);
	void drawLine(int x1, int y1, int x2, int y2);
	void drawRectangle(int x1, int y1, int x2, int y2);
	void drawDimensions();
	void drawRecords();
	void drawMouseRec();
    void init();    // initialize OpenGL states

    void setCamera(float posX, float posY, float posZ, float targetX, float targetY, float targetZ);
    void setViewport(int width, int height);
    void draw();
    
	void setMouseLeft(bool flag) { mouseLeftDown = flag; };
    void setMouseRight(bool flag) { mouseRightDown = flag; };
    void setMousePosition(int x, int y);
	void setMouseRecPos(int x,int y){ mouseRecX = x; mouseRecY = y; }; // mouse drag start position

    void setBackgroundRed(float value);             // change background colour
    void setBackgroundGreen(float value);
    void setBackgroundBlue(float value);

	void setRecordAlpha(float value);
	void resetDimension(int i);
	void updateDimension();
	void changeDimPosition(int i ,int j);
	void resetDimPos();
	void reorder_clutter();
	void PCA();

	//void loadFile();
	bool activated;
	int pom_dim;
	double vzd_dim;
	int dim_size, record_size;
	bool opened;
	bool drawing;
	int windowWidth;
    int windowHeight;

	//DIMENSIONS
	float *max_dim;
	float *min_dim;
	std::string *name_dim;
	int *dim_number;
	int *dim_pos_x;
	int *dim_pos_x_init;

	//RECORDS
	bool *active_records;
	int poc_pridanych;
	double **records;

	//Clutter analysis
	void setThreshold(float t);
	int outlinersCount(int dim1, int dim2);
	bool isOutliner(int record_number, int dim_number1, int dim_number2);
	//void drawOutliners();
	void fillOutliners();
	double* threshold;
	//int** Matrix;
	bool** outliner_record;
	int** permutation;

	int* outliner_numbers;

	int perm_pocet;
	void permute(int array[], const int start, const int n);

	//PCA analysis
	double* mean; // (x1+x2+...xn) /n
	double** ccv; // covariance matrix
	double getMax(double **a,int dim);
	double getMin(double **a,int dim);
	//BluebitMatrix31::IMatrix^ ccv;
	double covariance(int d1, int d2);
	int* dim_order_new;
	double* eigenvalues;
	
protected:

private:
    // member functions
    void initLights();                              // add a white light ti scene
    // members
   
    bool animateFlag;                               // on/off animation
    bool changeDrawMode;
    int drawMode;
    unsigned int listId;                            // display list ID
    bool mouseLeftDown;
    bool mouseRightDown;
    int mouseX;
    int mouseY;
	
	int mouseRecX;
	int mouseRecY;

    float cameraAngleX;
    float cameraAngleY;
    float cameraDistance;
    float bgColor[4];
    bool bgFlag;
    unsigned char* frameBuffer;                     // framebuffer to store RGBA color
    int bufferSize;                                 // framebuffer size in bytes

	float record_alpha;
	
};
#endif
