/**************************************************************************************************/
//	permute()
//	
//	Template function for permuting an array of n elements.  Works with any 
//	builtin data type or class for which the assignment operator is overloaded.
//	Maintains a lexicographic (dictionary-type) ordering based on the order in the
//	initial array.
//	
//	Arguments:
//			array - an array of builtin datatypes or objects
//			start - the initial point at which to start the permutation (allows for
//					the permutation of the last X elements of the array, and not 
//					just all of the elements).
//			n -		the total number of elements in the array
//	
//	Example call:
//	char array[]="bar";
//	permute(array,0,3);
//
//		output:
//				b a r 
//				b r a 
//				a b r 
//				a r b 
//				r b a 
//				r a b 
//	
//	Reinterpretted, simplified, and templatized version of an algorithm from :
//	Flamig, B. 1995.  Practical Algorithms in C++.

#include <iostream>

template <typename T>
T array[] permute(T array[], const int start, const int n) {
	//	display the current array
	for (int i=0; i<n; ++i)
		std::cout << array[i] << " ";
	std::cout << std::endl;
	
	//	start the permutation loop
	if (start < n) {
		int i, j;
		for (i = n-2; i >= start; i--) {
			for (j = i + 1; j < n; j++) {
				//	swap values in positions i and j
				T temp=array[i];
				array[i]=array[j];
				array[j]=temp;
				
				//	perform recursive call to permute
				permute(array, i+1, n);
			}
			
			//	rotate the elements between i and the end of the array
			T tmp = array[i];
			for (int k = i; k < n-1; k++) {
				array[k] = array[k+1];
			}
			array[n-1] = tmp;
		}
	}
}